/*
 * ctrl.h
 *
 * Header file for the control message routines.
 *
 * Kekoa Proudfoot 12/30/96
 */

#ifndef CTRL_H_DEFINED
#define CTRL_H_DEFINED

/*****************************************************************************
 * Include files
 *****************************************************************************/

#include <stdio.h>

/*****************************************************************************
 * Type definitions
 *****************************************************************************/

/* Enum of control messages op types */

typedef enum ctrl_op_t { 

    /* Shared ops */

    CTRL_BAD              = (unsigned char)0x00,

    /* Client to server control ops */

    CTRL_REQ_CONNECT      = (unsigned char)0x01,
    CTRL_REQ_SERVER_INFO  = (unsigned char)0x02,
    CTRL_REQ_PLAYER_INFO  = (unsigned char)0x03,
    CTRL_REQ_RULE_INFO    = (unsigned char)0x04,

    /* Server to client ops */

    CTRL_REP_ACCEPT       = (unsigned char)0x81,
    CTRL_REP_REJECT       = (unsigned char)0x82,
    CTRL_REP_SERVER_INFO  = (unsigned char)0x83,
    CTRL_REP_PLAYER_INFO  = (unsigned char)0x84,
    CTRL_REP_RULE_INFO    = (unsigned char)0x85
} ctrl_op_t;    

/* Control message structs */

typedef struct ctrl_req_connect_t {
    char  op;
    char *game;
    char  net_version;
} ctrl_req_connect_t;

typedef struct ctrl_req_server_info_t {
    char  op;
    char *game;
    char  net_version;
} ctrl_req_server_info_t;

typedef struct ctrl_req_player_info_t {
    char  op;
    char  player;
} ctrl_req_player_info_t;

typedef struct ctrl_req_rule_info_t {
    char  op;
    char *rule;
} ctrl_req_rule_info_t;

typedef struct ctrl_rep_accept_t {
    char   op;
    unsigned short  port;
    short  unknown;
} ctrl_rep_accept_t;

typedef struct ctrl_rep_reject_t {
    char  op;
    char *reason;
} ctrl_rep_reject_t;

typedef struct ctrl_rep_server_info_t {
    char  op;
    char *address;
    char *host;
    char *map;
    char  num_clients;
    char  max_clients;
    char  net_version;
} ctrl_rep_server_info_t;

typedef struct ctrl_rep_player_info_t {
    char  op;
    char  player;
    char *name;
    char  shirt_color;
    char  pants_color;
    char  unknown[3];
    long  frags;
    long  time;
    char *address;
} ctrl_rep_player_info_t;

typedef struct ctrl_rep_rule_info_t {
    char  op;
    char *rule;
    char *value;
} ctrl_rep_rule_info_t;

/* Union of all client to server control message structs */

typedef union cl_to_sv_ctrl_t {
    char                     op;
    ctrl_req_connect_t       req_connect;
    ctrl_req_server_info_t   req_server_info;
    ctrl_req_player_info_t   req_player_info;
    ctrl_req_rule_info_t     req_rule_info;
} cl_to_sv_ctrl_t;

/* Union of all server to client control message structs */

typedef union sv_to_cl_ctrl_t {
    char                     op;
    ctrl_rep_accept_t        rep_accept;
    ctrl_rep_reject_t        rep_reject;
    ctrl_rep_server_info_t   rep_server_info;
    ctrl_rep_player_info_t   rep_player_info;
    ctrl_rep_rule_info_t     rep_rule_info;
} sv_to_cl_ctrl_t;

/*****************************************************************************
 * Interface function declarations
 *****************************************************************************/

extern int   cl_to_sv_ctrl_get  (char *buf, cl_to_sv_ctrl_t *ctrl);
extern int   cl_to_sv_ctrl_put  (char *buf, cl_to_sv_ctrl_t *ctrl);
extern char *cl_to_sv_ctrl_name (cl_to_sv_ctrl_t *ctrl);
extern void  cl_to_sv_ctrl_debug(cl_to_sv_ctrl_t *ctrl, FILE *file);
extern char  cl_to_sv_ctrl_peek (char *buf);

extern int   sv_to_cl_ctrl_get  (char *buf, sv_to_cl_ctrl_t *ctrl);
extern int   sv_to_cl_ctrl_put  (char *buf, sv_to_cl_ctrl_t *ctrl);
extern char *sv_to_cl_ctrl_name (sv_to_cl_ctrl_t *ctrl);
extern void  sv_to_cl_ctrl_debug(sv_to_cl_ctrl_t *ctrl, FILE *file);
extern char  sv_to_cl_ctrl_peek (char *buf);

#endif
