#ifndef MAIN_H_DEFINED
#define MAIN_H_DEFINED

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <ctype.h>
#include <time.h>
#include <signal.h>
#include <errno.h>
#include <sys/types.h>

#ifndef WIN32
#include <unistd.h>
#include <sys/socket.h> 
#include <sys/param.h> 
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#endif

#include "regex.h"
#include "packet.h"
#include "mesg.h"
#include "ctrl.h"
#include "queue.h"
#include "timer.h"
#include "debug.h"
#include "net.h"
#include "hexdump.h"

#ifndef TRUE
#define TRUE 1
#endif

#ifndef FALSE
#define FALSE 0
#endif

#define WHITE                       0

#define SECS_BETWEEN_RETRYS         60
#define POLL_TIMEOUT                1.0
#define PASSWORD_TIMEOUT            30
 
#define BAN_WAIT                    10

#define NOT_CONNECTED               0
#define CONNECTED                   1
#define IN_THE_GAME                 2
 
#define MAX_COMMAND_LEN             2048
#define MAX_STRING_LEN              256
#define MAX_PLAYERS                 16
#define MAX_NAMELEN                 15
#define MAX_IPADDR_LEN              15
#define MAX_CLIENTS                 10

struct player_struct {
    int                        valid;
    int                        just_entered;
    unsigned char              raw_mapped_name[(MAX_NAMELEN*4)+1];
    unsigned char              real_name[MAX_NAMELEN+1];
    char                       name[MAX_NAMELEN+1];
    char                       ipaddr[MAX_IPADDR_LEN+1];
    char                       time[MAX_IPADDR_LEN+1];
    int                        seconds;
    int                        shirt;
    int                        pants;
    int                        frags;
};

struct client_struct {
    int                        valid;
    int                        password;
    qtimer_t                   password_timer;
    char                       realname[MAX_STRING_LEN];
    unsigned char              stagename[MAX_STRING_LEN];
    int                        socket;
    struct sockaddr_in         addr;
};  

struct stuff_command {
    char                      *command;
    struct stuff_command      *next;
};

struct ban_item {
    int                       type;
    char                     *ipaddr;
    unsigned char            *regex;
    regex_t                   regcmp;
    int                       shirt;
    int                       pants;
    char                     *banner;
    char                     *bannee;
    struct ban_item          *next;
};

struct global_state { 
    FILE                      *logfp;
    FILE                      *statslogfp;
    char                       configfile[MAX_STRING_LEN];

    int                        kick_impulse;
    int                        ban_impulse;
    int                        init_impulse;
    int                        init01_impulse;
    int                        init02_impulse;
    int                        init03_impulse;
    int                        init04_impulse;
    int                        ban_white_guys;
    int                        allow_null_name;
    int                        allow_in_game_command;
    char                       password[MAX_STRING_LEN];
    char                       banfile[MAX_STRING_LEN];

    struct player_struct       players[MAX_PLAYERS];
    struct player_struct       players_old[MAX_PLAYERS];
    char                       mapname[MAX_STRING_LEN];
    int                        version;
    int                        max_players;
    struct stuff_command      *stuff_commands;

    int                        ban_lock;
    struct ban_item           *banip_list;
    struct ban_item           *banname_list;
    struct ban_item           *bancolor_list;
    struct ban_item           *banexclude_list;

    char                       player_name[MAX_NAMELEN+1];
    int                        player_shirt;
    int                        player_pants;
    int                        player_number;

    int                        connect_state;
    qtimer_t                   retry_timer;
    qtimer_t                   pollinput_timer;
    char                       timestr[MAX_STRING_LEN];
 
    int                        server_socket;
    struct sockaddr_in         server_addr;
    int                        server_port;
    int                        server_personal_port;
    char                       server_name[MAX_STRING_LEN];

    float                      time;
    int                        keepalive_count;
    int                        signon_level;
    int                        post_signon;

    qqueue_t                   send_reliable_queue;
    qqueue_t                   send_unreliable_queue;
    unsigned int               send_reliable_number;
    unsigned int               send_unreliable_number;
    packet_t                   filler_packet;

    packet_t                   recv_packet;
    packet_t                   recv_buffer;
    int                        recv_buffer_pos;
    unsigned int               recv_reliable_number;
    unsigned int               recv_unreliable_number;

    int                        accepting_clients;
    struct sockaddr_in         listen_addr;
    int                        listen_socket;
    int                        listen_port;
    struct client_struct       client[MAX_CLIENTS];
    char                       print_buf[MAX_COMMAND_LEN];
    char                       stuff_buf[MAX_COMMAND_LEN];
    char                       client_recv_buffer[MAX_COMMAND_LEN];
    char                       client_send_buffer[MAX_COMMAND_LEN];
} gs;

/*****************************************************************************
 * Functions dealing with the top level - found in main.c
 *****************************************************************************/
extern void  run_main_loop(void);
extern void  command_line_process(int, char **);
extern void  usage(void);
extern void  print_banner(void);
extern void  init_signal_handlers(void);
extern void  sig_catcher(int);
extern void  shutdown_all(void);
extern void  init_time(void);
extern void  do_timeout_ops(void);
extern void  print_to_log(char *, ...);
extern void  print_to_stats_log(char *, ...);

/*****************************************************************************
 * Functions dealing with the config file - found in config.c
 *****************************************************************************/
extern void  read_config_file(void);

/*****************************************************************************
 * Functions dealing with stuff commands - found in stuff.c
 *****************************************************************************/
extern void  destroy_stuff_commands(void);
extern void  add_stuff_command(char *);

/*****************************************************************************
 * Functions to connect with the Quake Server - found in qconnect.c
 *****************************************************************************/
extern int   is_connected(void);
extern int   is_in_the_game(void);
extern void  init_quake_connection(void);
extern void  reset_connection(void);   
extern void  empty_send_queues(void);   
extern void  connect_quake_server(void);

/*****************************************************************************
 * Functions to recv info from the quake server - found in qrecv.c
 *****************************************************************************/
extern void  process_control_packet(void);
extern void  process_server_packet(void);
extern int   process_mesg(char *);
extern void  signon_protocol(void);
extern void  parse_mesg_prints(char *);
extern void  process_special_admin_commands(char *);
extern int   check_for_say_print(char *);
extern void  print_delta_status(void);
extern void  get_banner_from_cmd(char *, char *);
extern char *map_weird_chars(unsigned char *);
extern char *map_nl_to_semi(unsigned char *);
extern void  raw_map_name(unsigned char *, unsigned char *);

/*****************************************************************************
 * Functions to send info to the quake server - found in qsend.c
 *****************************************************************************/
extern void  init_filler_packet(void);
extern void  send_packet(packet_t *);   
extern void  send_unreliable_head(void);   
extern void  send_reliable_packet(packet_t *);   
extern void  send_reliable_head(void);   
extern void  send_disconnect(void);   
extern void  send_connection_request(void);   
extern void  send_keepalive(void);   
extern void  send_impulse(int);   
extern void  send_fire(void);   
extern void  send_jump(void);   
extern void  send_mesg_client_connect(void);   
extern void  send_mesg_signon_2(void);   
extern void  send_name_command(char *);
extern void  send_console_command(char *, ...);   
extern void  send_acknowledge(void);

/*****************************************************************************
 * Functions to communicate with clients - found in client.c
 *****************************************************************************/
extern void  process_client_command(char *, int);
extern void  check_client_password(int, char *);
extern void  init_client_connections(void);
extern void  open_listen_connection(void);
extern void  accept_new_client(void);
extern void  disconnect_client(int);
extern void  disconnect_all_clients(void); 
extern void  recv_client_message(int);
extern void  send_to_clients(char *, ...);
extern void  send_to_client(int, char *, ...);
extern void  _send_to_client(int, char *, va_list);
extern void  send_client_init_game_info(int);

/*****************************************************************************
 * Functions to deal with banning - found in ban.c
 *****************************************************************************/
extern void     destroy_ban_lists(void);
extern void     destroy_ban_item(struct ban_item *);
extern struct   ban_item *make_ban_item(void);
extern int      map_ip_to_regex(char *, char *);
extern void     add_ban_ip_item(char *, char *, char *);
extern void     add_ban_exclude_item(char *, char *, char *);
extern void     add_ban_name_item(char *, char *, char *);
extern void     add_ban_color_item(int, int);
extern void     remove_ban_item_by_num(int);
extern void     remove_banip_item_by_regex(char *);
extern void     remove_banexclude_item_by_regex(char *);
extern void     remove_banname_item_by_regex(char *);
extern void     remove_bancolor_item_by_color(int, int);
extern void     send_ban_list(int);
extern void     say_ban_list(void);
extern int      ipaddr_is_excluded(char *);
extern void     ban_check(void);
extern int      ban_ok(void);
extern void     map_control_chars(unsigned char *, unsigned char *, int);
extern void     read_ban_file(void);
extern void     write_ban_file(void);

#endif
